package net.wanji.utc.service.control.impl;

import lombok.RequiredArgsConstructor;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.databus.dto.ExtendPhaseDTO;
import net.wanji.databus.dto.StepingPhaseDTO;
import net.wanji.databus.vo.*;
import net.wanji.feign.service.common.FeignCommon;
import net.wanji.utc.cache.CrossInfoCache;
import net.wanji.utc.cache.UtcFeignClientCache;
import net.wanji.utc.common.constant.Constants;
import net.wanji.utc.po.hk.request.DelBaseConfigPO;
import net.wanji.utc.service.control.ControlCommandService;
import net.wanji.utc.util.HttpRestUtil;
import net.wanji.utc.util.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * @author duanruiming
 * @date 2023/01/12 13:18
 */
@Service("wanJiControlCommandService")
@RequiredArgsConstructor
public class WanJiControlCommandServiceImpl implements ControlCommandService {

    private static final Logger log = LoggerFactory.getLogger(WanJiControlCommandServiceImpl.class);
    private final HttpRestUtil httpRestUtil;
    @Value("url")
    private String rpcUri;
    private final UtcFeignClientCache utcFeignClientCache;
    private final CrossInfoCache crossInfoCache;

    /**
     * @param schemeSendVO
     * @return
     * @throws Exception
     */
    @Override
    public JsonViewObject schemeSend(SchemeSendVO schemeSendVO) throws Exception {
        String manufacturerIdCode = crossInfoCache.getManufacturerCodeByCrossId(schemeSendVO.getCrossCode());
        JsonViewObject jsonViewObject = utcFeignClientCache.getUtcFeignClientService(manufacturerIdCode).schemeSend(schemeSendVO);
        if (StringUtils.endsWithIgnoreCase("success", jsonViewObject.getStatus())) {
            return jsonViewObject.success("万集标准协议下发方案成功！");
        }
        return jsonViewObject;
    }

    @Override
    public JsonViewObject planSend(PlanSendVO planSendVO) throws Exception {
        String manufacturerIdCode = crossInfoCache.getManufacturerCodeByCrossId(planSendVO.getCrossCode());
        JsonViewObject jsonViewObject = utcFeignClientCache.getUtcFeignClientService(manufacturerIdCode).planSend(planSendVO);
        if (StringUtils.endsWithIgnoreCase("success", jsonViewObject.getStatus())) {
            return jsonViewObject.success("万集标准协议下发计划成功！");
        }
        return jsonViewObject;
    }

    @Override
    public JsonViewObject scheduleSend(ScheduleSendVO scheduleSendVO) throws Exception {
        String manufacturerIdCode = crossInfoCache.getManufacturerCodeByCrossId(scheduleSendVO.getCrossCode());
        JsonViewObject jsonViewObject = utcFeignClientCache.getUtcFeignClientService(manufacturerIdCode).scheduleSend(scheduleSendVO);
        if (StringUtils.endsWithIgnoreCase("success", jsonViewObject.getStatus())) {
            return jsonViewObject.success("万集标准协议下发时间表成功！");
        }
        return jsonViewObject;
    }

    @Override
    public JsonViewObject lockControl(ControlCommandVO commandVO) throws Exception {
        String manufacturerIdCode = crossInfoCache.getManufacturerCodeByCrossId(commandVO.getCrossCode());
        LockControlVO lockControlVO = new LockControlVO();
        lockControlVO.setCrossCode(commandVO.getCrossCode());
        lockControlVO.setCommand(commandVO.getCommand());
        lockControlVO.setDuration(commandVO.getDuration());
        lockControlVO.setPhaseList(commandVO.getPhaseList());
        JsonViewObject jsonViewObject = utcFeignClientCache.getUtcFeignClientService(manufacturerIdCode).lockControl(lockControlVO);
        if (StringUtils.endsWithIgnoreCase("success", jsonViewObject.getStatus())) {
            return jsonViewObject.success("万集标准协议锁定控制成功！");
        }
        return jsonViewObject;
    }

    @Override
    public JsonViewObject stepControl(String crossId, Integer command, Integer stepNum) throws Exception {
        String manufacturerIdCode = crossInfoCache.getManufacturerCodeByCrossId(crossId);
        JsonViewObject jsonViewObject = utcFeignClientCache.getUtcFeignClientService(manufacturerIdCode).stepControl(crossId, command, stepNum);
        if (StringUtils.endsWithIgnoreCase("success", jsonViewObject.getStatus())) {
            return jsonViewObject.success("万集标准协议步进控制成功！");
        }
        return jsonViewObject;
    }

    /**
     * 全红黄闪关灯
     *
     * @param crossId     信号机编号
     * @param command     指令 1 开 0 关
     * @param commandType 命令类型
     * @return
     * @throws Exception
     */
    @Override
    public JsonViewObject setSignalControl(String crossId, Integer command, Integer commandType) throws Exception {
        String manufacturerIdCode = crossInfoCache.getManufacturerCodeByCrossId(crossId);
        FeignCommon utcFeignClientService = utcFeignClientCache.getUtcFeignClientService(manufacturerIdCode);
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        ControlCommandVO controlCommandVO = new ControlCommandVO();
        controlCommandVO.setCrossCode(crossId);
        controlCommandVO.setCommand(command);
        controlCommandVO.setDuration(999);
        if (Objects.equals(Constants.CLOSE_LIGHT_CONTROL, commandType)) {
            controlCommandVO.setControlType(Constants.CLOSE_LIGHT_CONTROL);
            jsonViewObject = utcFeignClientService.closeLightControl(controlCommandVO);
        }
        if (Objects.equals(Constants.YELLOW_LIGHT_CONTROL, commandType)) {
            controlCommandVO.setControlType(Constants.YELLOW_LIGHT_CONTROL);
            jsonViewObject = utcFeignClientService.closeLightControl(controlCommandVO);
        }
        if (Objects.equals(Constants.ALL_RED_CONTROL, commandType)) {
            controlCommandVO.setControlType(Constants.ALL_RED_CONTROL);
            jsonViewObject = utcFeignClientService.closeLightControl(controlCommandVO);
        }
        if (StringUtils.endsWithIgnoreCase("success", jsonViewObject.getStatus())) {
            return jsonViewObject.success("万集标准协议设置信号机控制模式成功！");
        }
        return jsonViewObject;
    }

    @Override
    public JsonViewObject recoverSchedule(String crossId) throws Exception {
        String manufacturerIdCode = crossInfoCache.getManufacturerCodeByCrossId(crossId);
        JsonViewObject jsonViewObject = utcFeignClientCache.getUtcFeignClientService(manufacturerIdCode).recoverSchedule(crossId);
        if (StringUtils.endsWithIgnoreCase("success", jsonViewObject.getStatus())) {
            return jsonViewObject.success("万集标准协议恢复时间表成功！");
        }
        return jsonViewObject;
    }

    @Override
    public JsonViewObject phaseTimingSend(PhaseTimingSendVO phaseTimingSendVO) throws Exception {
        String manufacturerIdCode = crossInfoCache.getManufacturerCodeByCrossId(phaseTimingSendVO.getCrossCode());
        JsonViewObject jsonViewObject = utcFeignClientCache.getUtcFeignClientService(manufacturerIdCode).recoverSchedule(phaseTimingSendVO.getCrossCode());
        if (StringUtils.endsWithIgnoreCase("success", jsonViewObject.getStatus())) {
            return jsonViewObject.success("万集标准协议下发相位配时成功！");
        }
        return jsonViewObject;
    }

    @Override
    public JsonViewObject phaseDiffSend() {
        return null;
    }

    @Override
    public JsonViewObject delBaseConfig(DelBaseConfigPO delBaseConfigPO) {
        return null;
    }

    @Override
    public JsonViewObject timing(String crossCode, Date date) throws Exception {
        String manufacturerIdCode = crossInfoCache.getManufacturerCodeByCrossId(crossCode);
        JsonViewObject jsonViewObject = utcFeignClientCache.getUtcFeignClientService(manufacturerIdCode).signalTiming(crossCode, date);
        if (StringUtils.endsWithIgnoreCase("success", jsonViewObject.getStatus())) {
            return jsonViewObject.success("万集标准协议校时成功!");
        }
        return jsonViewObject.fail("万集标准协议校时失败!");
    }

    @Override
    public JsonViewObject tempScheme(SchemeOptSendVO schemeOptSendVO) throws Exception {
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        //东土协议
           /* TempSchemeVO tempSchemeVO = (TempSchemeVO) obj;
            String manufacturerIdCode = crossInfoCache.getManufacturerCodeByCrossId(tempSchemeVO.getCrossCode());
            jsonViewObject = utcFeignClientCache.getUtcFeignClientService(manufacturerIdCode).tempScheme(tempSchemeVO);
        */
        //海信协议
        TempSchemeSendVO tempSchemeSendVO = getTempSchemeSendVO(schemeOptSendVO);
        String manufacturerIdCode = crossInfoCache.getManufacturerCodeByCrossId(tempSchemeSendVO.getCrossId());
        jsonViewObject = utcFeignClientCache.getUtcFeignClientService(manufacturerIdCode).tempSchemeDetail(tempSchemeSendVO);
        if (StringUtils.endsWithIgnoreCase("success", jsonViewObject.getStatus())) {
            return jsonViewObject.success("万集标准协议下发临时方案成功!");
        }
        return jsonViewObject.fail("万集标准协议下发临时方案失败!");
    }

    /**
     * @return net.wanji.databus.vo.TempSchemeSendVO
     * @Description 逻辑处理，拼接临时方案参数
     * @Param [schemeOptSendVO]
     **/
    private TempSchemeSendVO getTempSchemeSendVO(SchemeOptSendVO schemeOptSendVO) {
        TempSchemeSendVO tempSchemeSendVO = new TempSchemeSendVO();
        //路口号
        String crossCode = schemeOptSendVO.getCrossCode();
        tempSchemeSendVO.setCrossId(crossCode);
        List<SchemeSendVO.Pattern> patternList = schemeOptSendVO.getPatternList();
        if (Optional.ofNullable(patternList).isPresent()) {
            //现在系统只传一个方案
            SchemeSendVO.Pattern pattern = patternList.get(0);
            //相位差
            tempSchemeSendVO.setOffset(pattern.getOffset());
            //周期
            tempSchemeSendVO.setCycle(pattern.getCycle());
            //方案号
            tempSchemeSendVO.setPattern(pattern.getPatternNo());
            //优化模式，现在只支持一，绿信比
            tempSchemeSendVO.setType("1");
            //环
            List<SchemeSendVO.Pattern.Ring> rings = pattern.getRings();
            List<Integer> list = new ArrayList<>();
            for (SchemeSendVO.Pattern.Ring ring : rings) {
                //相位集合
                List<SchemeSendVO.Pattern.Ring.Phase> phaseList = ring.getPhaseList();
                for (SchemeSendVO.Pattern.Ring.Phase phase : phaseList) {
                    String redTime = phase.getRedTime();
                    String greenTime = phase.getGreenTime();
                    String yellowTime = phase.getYellowTime();
                    //绿信比=相位绿灯时间+相位全红时间+相位黄灯时间
                    int phaseTime = Integer.parseInt(greenTime) + Integer.parseInt(redTime) + Integer.parseInt(yellowTime);
                    list.add(phaseTime);
                }
            }
            //海信有16个相位，不足的补0
            if (list.size() < 16) {
                for (int i = list.size(); i < 16; i++) {
                    list.add(0);
                }
            }
            String split = list.toString().replaceAll("^\\[|\\]$", "");
            tempSchemeSendVO.setSplit(split);
        }
        return tempSchemeSendVO;
    }
    @Override
    public JsonViewObject tempSchemeDetail(TempSchemeSendVO tempSchemeSendVO) throws Exception {
        String manufacturerIdCode = crossInfoCache.getManufacturerCodeByCrossId(tempSchemeSendVO.getCrossId());
        JsonViewObject jsonViewObject = utcFeignClientCache.getUtcFeignClientService(manufacturerIdCode).tempSchemeDetail(tempSchemeSendVO);
        if (StringUtils.endsWithIgnoreCase("success", jsonViewObject.getStatus())) {
            return jsonViewObject.success("下发临时方案成功!");
        }
        return jsonViewObject.fail("下发临时方案失败!");
    }

    @Override
    public JsonViewObject extendPhase(ExtendPhaseDTO extendPhase) throws Exception {
        String manufacturerIdCode = crossInfoCache.getManufacturerCodeByCrossId(extendPhase.getCrossId());
        JsonViewObject jsonViewObject = utcFeignClientCache.getUtcFeignClientService(manufacturerIdCode).extendPhase(extendPhase);
        if (StringUtils.endsWithIgnoreCase("success", jsonViewObject.getStatus())) {
            return jsonViewObject.success("海信-延长相位成功!");
        }
        return jsonViewObject.fail("海信-延长相位失败!");
    }

    @Override
    public JsonViewObject stepingPhase(StepingPhaseDTO stepingPhaseDTO) throws Exception {
        String manufacturerIdCode = crossInfoCache.getManufacturerCodeByCrossId(stepingPhaseDTO.getCrossId());
        JsonViewObject jsonViewObject = utcFeignClientCache.getUtcFeignClientService(manufacturerIdCode).stepingPhase(stepingPhaseDTO);
        if (StringUtils.endsWithIgnoreCase("success", jsonViewObject.getStatus())) {
            return jsonViewObject.success("步进相位成功!");
        }
        return jsonViewObject.fail("步进相位失败!");
    }
}
