package net.wanji.utc.service.staticinfo.impl;

import lombok.RequiredArgsConstructor;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.databus.dao.entity.*;
import net.wanji.databus.dao.mapper.*;
import net.wanji.databus.dto.*;
import net.wanji.databus.po.CrossInfoPO;
import net.wanji.databus.po.CrossInfoPOExt;
import net.wanji.databus.po.ManufacturerInfoPO;
import net.wanji.databus.vo.PlanSectionVO;
import net.wanji.databus.vo.SchemePhaseLightsVO;
import net.wanji.utc.cache.CrossInfoCache;
import net.wanji.utc.common.typeenum.BasicEnum;
import net.wanji.utc.service.staticinfo.*;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * @author Kent HAN
 * @date 2022/11/15 10:32
 */
@Service
@RequiredArgsConstructor
public class StaticInfoServiceImpl implements StaticInfoService {
    private final HkCrossInfoService hkCrossInfoService;
    private final CrossInfoMapper crossInfoMapper;
    private final ManufacturerInfoMapper manufacturerInfoMapper;
    private final HkSchemePhaseLightsService hkSchemePhaseLightsService;
    private final HkPlanSectionService hkPlanSectionService;
    private final HkCrossSchedulesService hkCrossSchedulesService;
    private final CrossSchemeMapper crossSchemeMapper;
    private final CrossPhaseMapper crossPhaseMapper;
    private final CrossLightsMapper crossLightsMapper;
    private final CrossPhaseLightsMapper crossPhaseLightsMapper;
    private final CrossPlanMapper crossPlanMapper;
    private final CrossSectionMapper crossSectionMapper;
    private final CrossSchedulesMapper crossSchedulesMapper;
    private final WanJiCommonStaticInfoService wanjiCommonStaticInfoService;
    private final CrossInfoCache crossInfoCache;
    private final OthersStaticInfoService othersStaticInfoService;

    @Override
    public CrossInfoPO crossInfoById(String crossId) throws Exception {
        CrossInfoPOExt crossInfoPOExt = crossInfoMapper.selectById(crossId);
        CrossInfoPO crossInfoPO = new CrossInfoPO();
        BeanUtils.copyProperties(crossInfoPOExt, crossInfoPO);
        return crossInfoPO;
    }

    @Override
    @Transactional
    public List<CrossInfoPO> crossBasicInfo(CrossInfoDTO crossInfoVO) throws Exception {
        List<CrossInfoPO> crossInfoPOList = new ArrayList<>();
        if (Objects.equals(BasicEnum.ManufacturerEnum.HK.getCode(), crossInfoVO.getManufacturerCode())) {
            crossInfoPOList = hkCrossInfoService.hkCrossBasicInfo();
        } else {
            // todo 其他厂商
//            Integer manufacturerId = manufacturerInfoMapper.selectIdByCode(crossInfoVO.getManufacturerCode());
//            crossInfoPOList = crossInfoMapper.selectByManufacturerId(manufacturerId);
        }
        //存储信号机信息
        if (crossInfoPOList.size() == 0) {
            return null;
        }
        for (CrossInfoPO crossInfoPO : crossInfoPOList) {
            String code = crossInfoPO.getCode();
            Integer manufacturerId = crossInfoPO.getManufacturerId();
            String id = crossInfoMapper.selectIdByCodeAndManufacturerId(code, manufacturerId);
            if (id != null) {
                // 如果有记录，则更新
                crossInfoMapper.updateOneByCodeAndManufacturerId(crossInfoPO);
            } else {
                // 如果无记录，则新增
                crossInfoMapper.insertOne(crossInfoPO);
            }
        }
        String manufacturerCode = crossInfoVO.getManufacturerCode();
        ManufacturerInfoPO manufacturerInfoPO = manufacturerInfoMapper.selectByCode(manufacturerCode);
        Integer manufacturerId = manufacturerInfoPO.getId();
        crossInfoPOList = crossInfoMapper.selectByManufacturerId(manufacturerId);
        // 释放内存
        hkCrossInfoService.clearRes();
        return crossInfoPOList;
    }

    @Override
    @Transactional
    public void schemePhaseLights(SchemePhaseLightsDTO schemePhaseLightsDTO) throws Exception {
        String crossId = schemePhaseLightsDTO.getCrossId();
        // 获取厂商ID和信号机ID
        CrossInfoPO crossInfoPO = crossInfoMapper.selectByPrimaryKey(crossId);
        String crossCode = crossInfoPO.getCode();
        Integer manufacturerId = crossInfoPO.getManufacturerId();
        // 获取厂商代码
        ManufacturerInfoPO manufacturerInfoPO = manufacturerInfoMapper.selectById(manufacturerId);
        String manufacturerCode = manufacturerInfoPO.getCode();
        if (Objects.equals(BasicEnum.ManufacturerEnum.HK.getCode(), manufacturerCode)) {
            // 海康
            hkSchemePhaseLightsService.hkSchemePhaseLights(crossId, crossCode);
        } else {
            // todo 其他厂商
            // 默认万集标准服务
            wanjiCommonStaticInfoService.schemePhaseLights(crossId, crossCode);
        }
    }

    @Override
    @Transactional
    public void planSection(PlanSectionDTO planSectionDTO) throws Exception {
        String crossId = planSectionDTO.getCrossId();
        // 获取厂商ID和信号机ID
        CrossInfoPO crossInfoPO = crossInfoMapper.selectByPrimaryKey(crossId);
        String crossCode = crossInfoPO.getCode();
        String crossInfoId = crossInfoPO.getId();
        Integer manufacturerId = crossInfoPO.getManufacturerId();
        // 获取厂商代码
        ManufacturerInfoPO manufacturerInfoPO = manufacturerInfoMapper.selectById(manufacturerId);
        String manufacturerCode = manufacturerInfoPO.getCode();
        Integer planNo = planSectionDTO.getPlanNo();
        if (Objects.equals(BasicEnum.ManufacturerEnum.HK.getCode(), manufacturerCode)) {
            // 海康
            hkPlanSectionService.planSection(crossCode, planNo, crossId);
        } else {
            wanjiCommonStaticInfoService.planSection(crossInfoId, crossCode, planNo);
        }
    }

    @Override
    @Transactional
    public void crossSchedules(CrossSchedulesDTO crossSchedulesDTO) throws Exception {
//        String manufacturerCode = crossSchedulesDTO.getManufacturerCode();
        List<String> crossIdList = crossSchedulesDTO.getCrossIdList();
        String manufacturerCode = crossInfoCache.getManufacturerCodeByCrossId(crossIdList.get(0));
        if (Objects.equals(BasicEnum.ManufacturerEnum.HK.getCode(), manufacturerCode)) {
            // 海康
            hkCrossSchedulesService.crossSchedules(crossIdList);
        } else {
            if (!CollectionUtils.isEmpty(crossIdList)) {
                for (String crossId : crossIdList) {
                    CrossInfoPO crossInfoPO = crossInfoMapper.selectByPrimaryKey(crossId);
                    if (Objects.nonNull(crossInfoPO)) {
                        String signalIc = crossInfoPO.getCode();
                        wanjiCommonStaticInfoService.schedules(crossId, signalIc);
                    }
                }
            }
        }

    }

    @Override
    public SchemePhaseLightsVO buildSchemePhaseLightsResponse(SchemePhaseLightsDTO schemePhaseLightsDTO) {
        String crossId = schemePhaseLightsDTO.getCrossId();
        List<CrossSchemePO> CrossSchemePOList = crossSchemeMapper.selectByCrossId(crossId);
        List<CrossPhasePO> crossPhasePOList = crossPhaseMapper.selectByCrossId(crossId);
        List<CrossLightsPO> crossLightsPOList = crossLightsMapper.selectByCrossId(crossId);
        List<CrossPhaseLightsPO> crossPhaseLightsPOList = crossPhaseLightsMapper.selectByCrossId(crossId);


        net.wanji.databus.vo.SchemePhaseLightsVO schemePhaseLightsVO = new net.wanji.databus.vo.SchemePhaseLightsVO();
        schemePhaseLightsVO.setCrossSchemeList(CrossSchemePOList);
        schemePhaseLightsVO.setCrossPhaseList(crossPhasePOList);
        schemePhaseLightsVO.setCrossLightsList(crossLightsPOList);
        schemePhaseLightsVO.setCrossPhaseLightsPOList(crossPhaseLightsPOList);

        return schemePhaseLightsVO;
    }

    @Override
    public PlanSectionVO buildPlanSectionResponse(String crossId) {
        List<CrossPlanPO> crossPlanPOList = crossPlanMapper.selectByCrossId(crossId);
        List<CrossSectionPO> crossSectionPOList = crossSectionMapper.selectByCrossId(crossId);

        PlanSectionVO planSectionVO = new PlanSectionVO();
        planSectionVO.setCrossPlanPOList(crossPlanPOList);
        planSectionVO.setCrossSectionPOList(crossSectionPOList);

        return planSectionVO;
    }

    @Override
    public PlanSectionVO buildPlanSectionResponse(String crossId, Integer planNo) {
        List<CrossPlanPO> crossPlanPOList = crossPlanMapper.selectByCrossIdAndPlanNo(crossId, planNo);
        Integer planId = crossPlanMapper.selectIdByNo(crossId, planNo);
        List<CrossSectionPO> crossSectionPOList = crossSectionMapper.selectByCrossIdAndPlanId(crossId, planId);

        PlanSectionVO planSectionVO = new PlanSectionVO();
        planSectionVO.setCrossPlanPOList(crossPlanPOList);
        planSectionVO.setCrossSectionPOList(crossSectionPOList);

        return planSectionVO;
    }

    @Override
    public List<CrossSchedulesPO> buildCrossSchedulesResponse(List<String> crossIdList) {
        List<CrossSchedulesPO> crossSchedulesPOList = crossSchedulesMapper.selectByCrossIds(crossIdList);
        return crossSchedulesPOList;
    }

    @Override
    public JsonViewObject signalTime(String crossId) throws Exception {
        String manufacturerCode = crossInfoCache.getManufacturerCodeByCrossId(crossId);
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        if (Objects.equals(BasicEnum.ManufacturerEnum.HK.getCode(), manufacturerCode)) {
            jsonViewObject = othersStaticInfoService.signalTime(crossId);
        } else {
            jsonViewObject = wanjiCommonStaticInfoService.signalTime(crossId);
        }
        return jsonViewObject;
    }

    @Override
    public JsonViewObject crossSchemeNo(String crossId) throws Exception {
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        jsonViewObject = wanjiCommonStaticInfoService.crossSchemeNo(crossId);
        return jsonViewObject;
    }

    @Override
    public JsonViewObject crossSchemeRings(CrossSchemeRingsDTO crossSchemeRingsDTO) throws Exception {
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        jsonViewObject = wanjiCommonStaticInfoService.crossSchemeRings(crossSchemeRingsDTO);
        return jsonViewObject;
    }

    @Override
    public JsonViewObject schemeOptLog(CrossSchemeRingsDTO crossSchemeRingsDTO) throws Exception {
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        jsonViewObject = wanjiCommonStaticInfoService.schemeOptLog(crossSchemeRingsDTO);
        return jsonViewObject;
    }

    @Override
    public JsonViewObject selectSchemeByParams(QueryByCrossIdAndTimeDTO queryByCrossIdAndTimeDTO) throws Exception {
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        jsonViewObject = wanjiCommonStaticInfoService.selectSchemeByParams(queryByCrossIdAndTimeDTO);
        return jsonViewObject;
    }
}
